#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <catalina_hmi.h>
#include "db_vm.h"

#define CURSOR	1

#define DEL	0x7f

static int rows;
static int cols;

void VM_sysinit(int argc, char *argv[])
{
    rows = t_geometry();
    cols = rows >> 8;
    rows &= 0xff;
}

void VM_getline(char *buf, int size)
{
	int i = 0;
	while (i < size - 1) {
		int ch = k_wait();
		if (ch == '\r') {
			buf[i++] = '\n';
			VM_putchar('\n');
			break;
		}
		else if (ch == '\b' || ch == DEL) {
			if (i > 0) {
                int row = t_getpos(CURSOR);
                int col = row >> 8;
                row &= 0xff;
                if (col > 0) {
                    t_setpos(CURSOR, col - 1, row);
                    VM_putchar(' ');
                    t_setpos(CURSOR, col - 1, row);
				}
				--i;
			}
		}
		else {
			buf[i++] = ch;
			VM_putchar(ch);
		}
	}
	buf[i] = '\0';
}

void VM_vprintf(const char *fmt, va_list ap)
{
    char buf[80], *p = buf;
    vsprintf(buf, fmt, ap);
    while (*p != '\0')
        VM_putchar(*p++);
}

void VM_putchar(int ch)
{
	t_char(CURSOR, ch);
}

VMVALUE VM_getreg(int index)
{
    VMVALUE value = 0;
    switch (index) {
    case RG_CNT:
        value = _cnt();
        break;
    case RG_DIRA:
        value = _dira(0, 0);
        break;
    case RG_OUTA:
        value = _outa(0, 0);
        break;
    case RG_INA:
        value = _ina();
        break;
    case RG_CLKFREQ:
        value = _clockfreq();
        break;
    }
	return value;
}

void VM_setreg(int index, VMVALUE value)
{
    switch (index) {
    case RG_DIRA:
        _dirb(0xffffffff, value);
        break;
    case RG_OUTA:
        _outb(0xffffffff, value);
        break;
    }
}

int strcasecmp(const char *s1, const char *s2)
{
    while (*s1 != '\0' && (tolower(*s1) == tolower(*s2))) {
        ++s1;
        ++s2;
    }
    return tolower((unsigned char) *s1) - tolower((unsigned char) *s2);
}
